﻿-- -----------------------------------------------------
-- Database QuizFaber x MySQL : Update from v.1 to v.2 
-- -----------------------------------------------------
-- Project : QuizFaber
-- Licence : MIT
-- Author  : Luca Galli
-- Email   : info@quizfaber.com
-- -----------------------------------------------------

CREATE TABLE IF NOT EXISTS `QuizState` (
  `Code` INT NOT NULL,
  `Description` VARCHAR(30) NULL,
  PRIMARY KEY (`Code`))
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS `WebLog` (
  `ID` INT NOT NULL AUTO_INCREMENT,
  `UserIdentity` VARCHAR(255) NULL,
  `SeverityLevel` INT NULL,
  `DateCreated` DATETIME NULL,
  `Message` MEDIUMTEXT NULL,
  PRIMARY KEY (`ID`))
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS `WebSeverityLevel` (
  `Level` INT NOT NULL,
  `Description` VARCHAR(20) NULL)
ENGINE = InnoDB;

DELETE FROM WebSeverityLevel;
INSERT INTO WebSeverityLevel (Level,Description) VALUES (0,'Debug');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (1,'Info');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (2,'Warning');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (3,'Error');

DELETE FROM QuizState;
INSERT INTO QuizState (Code,Description) VALUES (0,'Draft');
INSERT INTO QuizState (Code,Description) VALUES (1,'Completed');
INSERT INTO QuizState (Code,Description) VALUES (2,'Candidate');
INSERT INTO QuizState (Code,Description) VALUES (10,'Published');
INSERT INTO QuizState (Code,Description) VALUES (98,'Archived');
INSERT INTO QuizState (Code,Description) VALUES (99,'Deleted');

ALTER TABLE Person ADD `Source` VARCHAR(1) NULL;
ALTER TABLE Quizzes ADD State INT NOT NULL DEFAULT 0;
ALTER TABLE Quizzes ADD Link VARCHAR(255) NULL;
ALTER TABLE Quizzes ADD Properties TEXT NULL;
ALTER TABLE Questions ADD QuestionName VARCHAR(256) NULL;
ALTER TABLE QuizResult ADD FinalPoints DOUBLE NULL;
ALTER TABLE QuizResult ADD ReviewPoints DOUBLE NULL;
ALTER TABLE QuizResult ADD NotAnsweredQuestsNum INT NULL;
ALTER TABLE QuizResult ADD NotValuatedQuestsNum INT NULL;
ALTER TABLE QuizResultQuestion ADD Points DOUBLE NULL;
ALTER TABLE QuizResultReport ADD Charset VARCHAR(255) NULL;
ALTER TABLE QuizResultReport ADD Language VARCHAR(10) NULL;
ALTER TABLE QuizResultReport ADD Origin MEDIUMTEXT NULL;

ALTER TABLE Person ADD CONSTRAINT FK_PersonRole FOREIGN KEY (UserRole) REFERENCES PersonRole(ID); 
ALTER TABLE Quizzes ADD CONSTRAINT FK_QuizState FOREIGN KEY (State) REFERENCES QuizState(Code);
ALTER TABLE Categories ADD CONSTRAINT FK_CategoryQuiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE ON UPDATE CASCADE; 
ALTER TABLE CustomQuestionTypes ADD CONSTRAINT FK_CustomQuestionType_Quiz FOREIGN KEY (QZ_ID) REFERENCES Quizzes(QZ_ID) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE QuizResultReport ADD CONSTRAINT FK_Report_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE QuizResult MODIFY COLUMN UserName VARCHAR(255) NOT NULL;
ALTER TABLE QuizResult MODIFY COLUMN UserLogin VARCHAR(255) NOT NULL;
ALTER TABLE QuizResult MODIFY COLUMN QuizName VARCHAR(255) NOT NULL;
ALTER TABLE QuizResult MODIFY COLUMN QuizTitle VARCHAR(1024) NOT NULL;
ALTER TABLE QuizResult ADD ReviewMark DOUBLE NULL;
ALTER TABLE QuizResult ADD ReviewDate DATETIME NULL;
ALTER TABLE QuizResult ADD LowestMark INT NOT NULL DEFAULT 0;
ALTER TABLE QuizResult ADD RoundMark DOUBLE NOT NULL DEFAULT 1;
ALTER TABLE QuizResult ADD IsDuplicated TINYINT NULL;
ALTER TABLE QuizResultQuestion ADD Feedback TEXT NULL;
ALTER TABLE QuizResultQuestion ADD IsCancelled TINYINT NULL;
ALTER TABLE QuizResultAnswer MODIFY COLUMN AdditionalText TEXT NULL;

-- ALTER TABLE QuizResultReport ADD CONSTRAINT FK_QuizResult FOREIGN KEY (ID) REFERENCES QuizResult(ID);
-- ALTER TABLE QuizResult ADD CONSTRAINT FK_Quiz FOREIGN KEY (QuizID) REFERENCES Quizzes(QZ_ID);

ALTER TABLE Settings ADD ServerName VARCHAR(50) NULL;
ALTER TABLE Settings ADD ServerDescription VARCHAR(255) NULL;
ALTER TABLE Settings ADD ServerVersion VARCHAR(50) NULL;
ALTER TABLE Settings ADD Environment VARCHAR(50) NULL;
ALTER TABLE Settings ADD PortalBaseUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD PhpMyAdminUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD OwnerUrl VARCHAR(255) NULL;
ALTER TABLE Settings ADD FooterText VARCHAR(255) NULL;

CREATE UNIQUE INDEX indexUserIdentity ON Person (UserIdentity);

-- password : admin
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Administrator' AS PersonName,'admin' AS UserIdentity,'21232f297a57a5a743894a0e4a801fc3' AS UserPassword,1 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'admin');
-- password : anonymous
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Anonymous' AS PersonName, 'anonymous@anonymous.nowhere' AS UserIdentity,'294de3557d9d00b3d2d8a1e6aab028cf' AS UserPassword,2 AS UserRole,1 AS IsEnabled) AS tmp WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'anonymous@anonymous.nowhere');

-- QuizSession
ALTER TABLE QuizSession DROP PRIMARY KEY;
UPDATE QuizSession SET SessionID = '0';
ALTER TABLE QuizSession MODIFY COLUMN SessionID INT PRIMARY KEY AUTO_INCREMENT;
ALTER TABLE QuizSession ADD CONSTRAINT FK_Person FOREIGN KEY (PersonID) REFERENCES Person(ID); 
ALTER TABLE QuizSession ADD CONSTRAINT FK_QuizResult FOREIGN KEY (QuizID) REFERENCES QuizResult(ID); 
ALTER TABLE QuizSession ADD UserAgent VARCHAR(255) NULL;

UPDATE Settings SET DbVersion = 2, DateModified=NOW(), ServerName='Node.js server', ServerDescription='Application for manage quizzes.',ServerVersion='4.1.0',Environment='DEV',PortalBaseUrl='http://localhost:3313/',OwnerUrl='https://www.quizfaber.com',FooterText='QuizFaber';
